﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Drawing.Imaging;
using System.IO;
using ThinkGeo.MapSuite.Core;
using Brushes = System.Drawing.Brushes;
using Pen = System.Drawing.Pen;
using PixelFormat = System.Drawing.Imaging.PixelFormat;

namespace Performance2
{
    public class CustomStyle : PointStyle
    {
        GeoImage _plannedReceiver;

        public CustomStyle()
        {
            Initialize();
        }
        
        private void Initialize()
        {
            const int size4 = 4;

            using (var pen = new Pen(Brushes.White))
            {
                using (var bmp = new Bitmap(size4, size4, PixelFormat.Format32bppArgb))
                {
                    using (Graphics gBmp = Graphics.FromImage(bmp))
                    {
                        gBmp.CompositingMode = CompositingMode.SourceCopy;
                        gBmp.CompositingQuality = CompositingQuality.HighQuality;
                        var b = Brushes.White;
                        gBmp.FillEllipse(b, 0, 0, size4, size4);
                        gBmp.DrawEllipse(pen, 0, 0, size4, size4);
                    }
                    _plannedReceiver = ToGeoImage(bmp);
                }
            }
        }

        private static GeoImage ToGeoImage(Image image)
        {
            var stream = new MemoryStream();
            image.Save(stream, ImageFormat.Png);
            stream.Seek(0L, SeekOrigin.Begin);
            return new GeoImage(stream);
        }
        
        protected override void DrawCore(IEnumerable<Feature> features, GeoCanvas canvas, Collection<SimpleCandidate> labelsInThisLayer,
                                         Collection<SimpleCandidate> labelsInAllLayers)
        {

            base.DrawCore(features, canvas, labelsInThisLayer, labelsInAllLayers);
            var gcanvas = canvas as GeoCanvas;

            if (gcanvas != null)
            {
                foreach (Feature feature in features)
                {
                    var pt = feature.Tag as PointShape;
                    GeoImage toDraw = null;
                    if (pt != null)
                    {
                        toDraw = _plannedReceiver;
                    }

                    if (toDraw != null)
                    {
                        var pointShape = feature.GetShape() as PointShape;
                        if (pointShape != null)
                        {
                            gcanvas.DrawWorldImageWithoutScaling(toDraw, pointShape.X, pointShape.Y, DrawingLevel.LevelOne);
                        }
                    }
                }
            }
        }

    }

}
