﻿using System.Collections.Generic;
using System.Windows;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WpfDesktopEdition;

namespace Performance2
{
    public partial class MainWindow
    {
        const string PointsOverlayName = "PointsOverLay";
        const string PointsLayerName = "PointsLayer";
        
        LayerOverlay _layerOverlay;
        InMemoryFeatureLayer _featureLayer;

        private BatchObservableCollection<Point> _receiverPoints;
        public BatchObservableCollection<Point> ReceiverPoints
        {
            get { return _receiverPoints ?? (_receiverPoints = new BatchObservableCollection<Point>()); }
            set { _receiverPoints = value; }
        }

        public MainWindow()
        {
            InitializeComponent();
        }

        private void WpfmapLoaded(object sender, RoutedEventArgs e)
        {
            InitializeData();

            var map = (WpfMap)sender;
            map.MapUnit = GeographyUnit.DecimalDegree;
            map.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.StandardColors.Black);

            _layerOverlay = new LayerOverlay()
            {
                TileType = TileType.SingleTile
            };

            //create in memory layer with point data
            _featureLayer = new InMemoryFeatureLayer { DrawingQuality = DrawingQuality.HighSpeed };

            _featureLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(new CustomStyle());
            _featureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            foreach (Point p in ReceiverPoints)
            {
                var lon = p.X;
                var lat = p.Y;
                var pointShape = new PointShape(lon, lat);
                var feature = new Feature(pointShape) { Tag = pointShape };
                _featureLayer.InternalFeatures.Add(feature);
            }

            _layerOverlay.Layers.Add(PointsLayerName, _featureLayer);
            map.Overlays.Add(PointsOverlayName, _layerOverlay);

            _featureLayer.FeatureSource.Open();
            map.CurrentExtent = _featureLayer.FeatureSource.GetBoundingBox();
            map.CenterAt(_featureLayer.GetBoundingBox().GetCenterPoint());
            _featureLayer.FeatureSource.Close();

            map.Refresh();
        }

        private void InitializeData()
        {
            const double startLat = 35.59526667;
            const double startLon = -99.55963611;
            const double latIncr = .00055;
            const double lonIncr = .00538;

            var curLon = startLon;

            var pts = new List<Point>();
            for (var i = 0; i < 50; i++)
            {
                var curLat = startLat;
                for (var j = 0; j < 500; j++)
                {
                    var p = new Point { Y = curLat, X = curLon };
                    pts.Add(p);
                    curLat += latIncr;
                }
                curLon += lonIncr;
            }
            ReceiverPoints.BatchReplace(pts);
        }
    }
}
